<?php

namespace Drupal\drd;

use Drupal\drd\Entity\DrdDomain;
use GuzzleHttp\Client;
use Psr\Http\Message\ResponseInterface;

/**
 * Class DrdHttpRequest.
 *
 * @package Drupal\drd
 */
class DrdHttpRequest {

  public static function getVersion() {
    $modules = system_rebuild_module_data();
    $version = $modules['drd']->info['version'];
    return empty($version) ? '8.x-3.x' : $version;
  }

  /**
   * @var DrdDomain
   */
  protected $domain;

  /**
   * @var string
   */
  protected $query = '';

  /**
   * @var array
   */
  protected $options = [];

  /**
   * @var ResponseInterface
   */
  protected $response;

  /**
   * @var int
   */
  protected $statusCode;

  /**
   * @var bool
   */
  protected $remoteIsDrd;

  /**
   * @param DrdDomain $domain
   * @return $this
   */
  public function setDomain($domain) {
    $this->domain = $domain;
    return $this;
  }

  /**
   * @param string $query
   * @return $this
   */
  public function setQuery($query) {
    $this->query = $query;
    return $this;
  }

  /**
   * @param string $key
   * @param string $value
   * @return $this
   */
  public function setOption($key, $value) {
    $this->options[$key] = $value;
    return $this;
  }

  /**
   * @return string|bool
   * @throws \Exception
   */
  public function getResponse() {
    if (empty($this->response)) {
      // We haven't received anything from remote
      return FALSE;
    }
    if (!$this->isRemoteDrd()) {
      // We received something from remote, but not from DRD remote
      throw new \Exception('Remote domain does not respond as DRD.');
    }

    // Let's decode the response from DRD remote
    $body = base64_decode($this->response->getBody()->getContents());
    if ($body === FALSE) {
      // Reponse can not be decoded
      throw new \Exception('Received unexpected content.');
    }

    // Return the actual response
    return $body;
  }

  /**
   * @return int
   */
  public function getStatusCode() {
    return $this->statusCode;
  }

  /**
   * @return bool
   */
  public function isRemoteDrd() {
    return $this->remoteIsDrd;
  }

  /**
   * @return void
   */
  public function request() {
    $url = $this->domain->buildUrl($this->query);
    $this->options['headers'] = $this->domain->getHeader();
    $this->options['headers']['X-Drd-Version'] = self::getVersion();
    $this->statusCode = -1;
    $this->remoteIsDrd = FALSE;
    try {
      $client = new Client(['base_uri' => $url->toUriString()]);
      $this->response = $client->request('get', NULL, $this->options);
      $this->statusCode = $this->response->getStatusCode();
      $this->remoteIsDrd = ($this->statusCode == 200 && $this->response->getHeaderLine('content-type') == 'text/plain; charset=utf-8');
    }
    catch (\Exception $ex) {
      $this->statusCode = $ex->getCode();
    }
  }

}
