<?php

/**
 * @file
 * Contains \Drupal\drd\Plugin\DrdActionRemoteBase.
 */

namespace Drupal\drd\Plugin;

use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Session\AccountInterface;
use Drupal\drd\Crypt\DrdCryptBase;
use Drupal\drd\DrdHttpRequest;
use Drupal\drd\DrdRemoteInterface;
use Drupal\drd\Entity\DrdDomain;

/**
 * Base class for DRD Remote Action plugins.
 */
abstract class DrdActionRemoteBase extends DrdActionBase {

  protected $response = FALSE;

  /**
   * @param string $id
   * @param DrdRemoteInterface $remote
   *
   * @return array|bool
   */
  public static function response($id, $remote, $arguments = []) {
    /** @var DrdActionBaseInterface $action */
    $action = self::instance($id);
    foreach ($arguments as $key => $value) {
      $action->setActionArgument($key, $value);
    }
    if ($action) {
      return $action->executeAction($remote);
    }
    return FALSE;
  }

  /**
   * Default remote action which can be overwritten by implementations.
   *
   * @param DrdDomain $domain
   * @return array|bool
   */
  public function executeAction($domain) {
    $this->response = FALSE;
    /** @var DrdDomain $domain */
    if (!$this->access($domain)) {
      return FALSE;
    }
    $this->remoteRequest();
    return $this->getResponse();
  }

  /**
   * {@inheritdoc}
   */
  public function access($domain, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $this->drdEntity = $domain;
    return $return_as_object ? new AccessResultAllowed() : TRUE;

    /**
     * TODO: during cron the current user is anonymous and that would let all
     * actions fail. Let's find out if there is a better way maybe using context
     * to find out if we are called during cron or otherwise.
    $access = $domain->status->access('edit', $account, TRUE)
      ->andIf($domain->access('update', $account, TRUE));

    return $return_as_object ? $access : $access->isAllowed();
    */
  }

  public function getResponse() {
    return $this->response;
  }

  protected function remoteRequest() {
    /** @var DrdDomain $domain */
    $domain = $this->drdEntity;
    $class = explode('\\', get_class($this));

    // Add authentication
    $args = [
      'auth' => $domain->get('auth')->value,
      'authsetting' => $domain->getAuthSetting(),
      'action' => array_pop($class),
    ] + $this->arguments;

    // Encrypt the arguments
    $method = $domain->get('crypt')->value;
    if (empty($method)) {
      return;
    }
    $crypt = DrdCryptBase::getInstance(
      $method,
      $domain->getCryptSetting()
    );

    $body = base64_encode(serialize([
      'uuid' => $domain->uuid(),
      'args' => $crypt->encrypt($args),
      'iv' => $crypt->getIV(),
    ]));

    /** @var DrdHttpRequest $request */
    $request = \Drupal::service('drd.http_request');
    $request->setDomain($domain)
      ->setQuery('drd-remote')
      ->setOption('body', $body)
      ->request();
    if (!$request->isRemoteDrd()) {
      return;
    }
    try {
      $this->response = $crypt->decrypt($request->getResponse(), $crypt->getIV());
    } catch (\Exception $ex) {
      return;
    }

    if (!empty($this->response['messages'])) {
      $domain->cacheRemoteMessages($this->response['messages']);
    }
    unset($this->response['messages']);

    if ($followup = $this->getFollowUpAction()) {
      \Drupal::service('queue.drd')->createItem(DrdActionBase::instance($followup), $domain);
    }
  }

}
